#!/usr/bin/env python3

'''
Purge ElasticSearch Data
========================
'''

import argparse
import json
import pycurl
import time
from datetime import date
from dateutil.relativedelta import relativedelta
from io import BytesIO
try:
    # python 3
    from urllib.parse import urlencode
except ImportError:
    # python 2
    from urllib import urlencode

parser = argparse.ArgumentParser(description='ElasticSearch Purge')
parser.add_argument('username')
parser.add_argument('password')
parser.add_argument('index')
parser.add_argument('retention')
parser.add_argument('mode')
parser.add_argument('--timefield', action='store', default='syslog_time',
                    help='Name of the time field in this index')
parser.add_argument('--timemode', action='store_true', default=False,
                    help='Time field as integer (default: False)')
parser.add_argument('--dateblock', action='store', default=1,
                    help='Splitting the index name by "-", which part contains the date (default: 1)')
args = parser.parse_args()


def getIndex():
    buffer = BytesIO()
    c = pycurl.Curl()
    c.setopt(c.URL, 'http://%s:%s@localhost:9200/_cat/indices?v&pretty&s=index&format=json' % (args.username, args.password))
    c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
    c.setopt(c.WRITEDATA, buffer)
    c.perform()
    c.close()
    return json.loads(buffer.getvalue())


def getAliases():
    buffer = BytesIO()
    c = pycurl.Curl()
    c.setopt(c.URL, 'http://%s:%s@localhost:9200/_cat/aliases?v&pretty&s=index&format=json' % (args.username, args.password))
    c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
    c.setopt(c.WRITEDATA, buffer)
    c.perform()
    c.close()
    return  json.loads(buffer.getvalue())


def purgeSingle(cat, day):
    for index in cat:
        if index['index'] == args.index:
            if args.timemode:
                datevalue = int(time.mktime(day.timetuple())) * 1000
            else:
                datevalue = day.strftime('%d/%b/%Y:%H:%M:%S +0000')
            post_data = {"query":{"bool":{"filter":{"range":{args.timefield:{"lte":datevalue}}}}}}
            postfields = json.dumps(post_data)
            print('  Postfields: ' + postfields)
            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s/_delete_by_query' % (args.username, args.password, args.index))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.POSTFIELDS, postfields)
            c.perform()
            c.close()
            print('  Delete: ' + buffer.getvalue().decode('utf8'))

            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s/_forcemerge?only_expunge_deletes=true' % (args.username, args.password, args.index))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.POSTFIELDS, '')
            c.perform()
            c.close()
            print('  Purge: ' + buffer.getvalue().decode('utf8'))
            return

    print('Index not found')


def purgeDaily(cat, day):
    block = int(args.dateblock)
    limit = day.strftime('%Y.%m.%d')
    for index in cat:
        parts = index['index'].split('-')
        if parts[0] == args.index and parts[block] < limit:
            print('  Index: ' + index['index'])
            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s' % (args.username, args.password, index['index']))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.CUSTOMREQUEST, 'DELETE')
            c.perform()
            c.close()
            print('  Delete: ' + buffer.getvalue().decode('utf8'))


cat = getIndex()
for alias in getAliases():
    if alias['alias'] == args.index:
        args.index = alias['index']
        break
day = date.today() + relativedelta(days=-int(args.retention))
print('Purge index %s older than %s' % (args.index, day))

if args.mode == 'single':
    purgeSingle(cat, day)
elif args.mode == 'daily':
    purgeDaily(cat, day)
else:
    print('Unknown mode')
